#ifndef UTILS_H
#define UTILS_H

/*****************************************************************************
 *                                 jugCLer                                   *
 *                 realtime raytracing experiment: utilities                 *
 *                                                                           *
 *  Copyright (C) 2013  Holger Bettag               (hobold@vectorizer.org)  *
 *                                                                           *
 *  This program is free software; you can redistribute it and/or modify     *
 *  it under the terms of the GNU General Public License as published by     *
 *  the Free Software Foundation; either version 2 of the License, or        *
 *  (at your option) any later version.                                      *
 *                                                                           *
 *  This program is distributed in the hope that it will be useful,          *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 *  GNU General Public License for more details.                             *
 *                                                                           *
 *  You should have received a copy of the GNU General Public License along  *
 *  with this program; if not, write to the Free Software Foundation, Inc.,  *
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.              *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#ifdef __linux__
#include <GL/glut.h>
#else
#include <GLUT/glut.h>
#endif

#ifdef WIN32
  #include <windows.h>
  #include <time.h>
#else    // unix
  #include <sys/time.h>
#endif

/*****************************
 *  simple time measurement  *
 *****************************/
// wraps system specific timer function
// returns number of seconds since an unspecified reference
double getSomeWallClockTime(void);


/*******************
 *  simple bitmap  *
 *******************/
struct PixelRGBA8888 {
  unsigned char r, g, b, a;
};

struct Bitmap {
  PixelRGBA8888* pixels;
  int width;
  int height;
  
  Bitmap(int Width, int Height) {
    width = Width;
    height = Height;
    pixels = new PixelRGBA8888[Width * Height];
    if (pixels == NULL) {
      fprintf(stderr,"out of memory for pixels\n");
      exit(EXIT_FAILURE);
    }
  }
  
  ~Bitmap() {
    delete[] pixels;
  }

  void draw(void) {
    glRasterPos2i(-1, -1);
    glDrawPixels(width, height, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
  }
};

#endif    // UTILS_H
